@@ -1,7 +1,6 @@
-/* SPDX-FileCopyrightText: 2020-2021 Tobias Leupold <tobias.leupold@gmx.de>
-
-   SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-KDE-Accepted-GPL
-*/
+// SPDX-FileCopyrightText: 2020-2022 Tobias Leupold <tl at stonemx dot de>
+//
+// SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
 // Local includes
 #include "ImagesListView.h"
@@ -21,6 +20,9 @@
 #include <QDebug>
 #include <QKeyEvent>
 #include <QMenu>
+#include <QDesktopServices>
+#include <QClipboard>
+#include <QRegularExpression>
 
 // C++ includes
 #include <algorithm>
@@ -107,6 +109,9 @@ ImagesListView::ImagesListView(KGeoTag::ImagesListType type, SharedObjects *shar
     connect(m_assignToMapCenter, &QAction::triggered,
             this, std::bind(&ImagesListView::assignToMapCenter, this, this));
 
+    m_assignToClipboard = m_contextMenu->addAction(i18n("Set coordinates from clipboard"));
+    connect(m_assignToClipboard, &QAction::triggered, this, &ImagesListView::assignToClipboard);
+
     m_assignManually = m_contextMenu->addAction(i18n("Set coordinates manually"));
     m_assignManually->setIcon(QIcon::fromTheme(QStringLiteral("add-placemark")));
     connect(m_assignManually, &QAction::triggered,
@@ -147,6 +152,12 @@ ImagesListView::ImagesListView(KGeoTag::ImagesListType type, SharedObjects *shar
     connect(m_removeImages, &QAction::triggered,
             this, std::bind(&ImagesListView::removeImages, this, this));
 
+    m_contextMenu->addSeparator();
+
+    m_openExternally = m_contextMenu->addAction(i18n("Open with default viewer"));
+    m_openExternally->setIcon(QIcon::fromTheme(QStringLiteral("file-zoom-in")));
+    connect(m_openExternally, &QAction::triggered, this, &ImagesListView::openExternally);
+
     connect(this, &QListView::customContextMenuRequested, this, &ImagesListView::showContextMenu);
 }
 
@@ -276,6 +287,7 @@ void ImagesListView::showContextMenu(const QPoint &point)
     m_automaticMatchingMenu->setEnabled(anySelected);
     m_bookmarksMenu->setEnabled(anySelected);
     m_assignToMapCenter->setEnabled(anySelected);
+    m_assignToClipboard->setEnabled(anySelected);
     m_assignManually->setEnabled(anySelected);
     m_editCoordinates->setEnabled(anySelected);
     m_lookupElevation->setEnabled(anySelected);
@@ -285,6 +297,7 @@ void ImagesListView::showContextMenu(const QPoint &point)
     if (anySelected) {
         m_removeImages->setText(i18np("Remove image", "Remove images", allSelected));
     }
+    m_openExternally->setEnabled(allSelected == 1);
 
     int hasCoordinates = 0;
     int changed = 0;
@@ -319,3 +332,63 @@ void ImagesListView::selectImages(bool coordinatesSet)
         }
     }
 }
+
+void ImagesListView::openExternally()
+{
+    // selectedPaths() always contains exactly one entry when this is called,
+    // because the connected action is only enabled in this very case
+    QDesktopServices::openUrl(QUrl::fromLocalFile(selectedPaths().first()));
+}
+
+void ImagesListView::assignToClipboard()
+{
+    const auto data = QGuiApplication::clipboard()->text().simplified();
+    bool dataParsed = false;
+    double lon = 0.0;
+    double lat = 0.0;
+
+    QRegularExpression re;
+    QRegularExpressionMatch match;
+
+    // Google Maps
+    // Schema: xx.xxxxxxxxxxxxxx, xx.xxxxxxxxxxxxx
+    if (! dataParsed) {
+        re = QRegularExpression(QStringLiteral("^(\\d+\\.\\d+), (\\d+\\.\\d+)$"));
+        match = re.match(data);
+        if (match.hasMatch()) {
+            bool lonOkay = false;
+            bool latOkay = false;
+            lat = match.captured(1).toDouble(&latOkay);
+            lon = match.captured(2).toDouble(&lonOkay);
+            dataParsed = latOkay && lonOkay;
+        }
+    }
+
+    // OpenStreetMap Geo-URI
+    // Schema: geo:xx.xxxxx,xx.xxxxx?z=xx
+    if (! dataParsed) {
+        re = QRegularExpression(QStringLiteral("^geo:(\\d+\\.\\d+),(\\d+\\.\\d+)\\?z=\\d+$"));
+        match = re.match(data);
+        if (match.hasMatch()) {
+            bool latOkay = false;
+            bool lonOkay = false;
+            lat = match.captured(1).toDouble(&latOkay);
+            lon = match.captured(2).toDouble(&lonOkay);
+            dataParsed = latOkay && lonOkay;
+        }
+    }
+
+    if (dataParsed) {
+        emit assignTo(selectedPaths(), Coordinates(lon, lat, 0.0, true));
+    } else {
+        emit failedToParseClipboard();
+    }
+}
+
+void ImagesListView::highlightImage(const QModelIndex &index)
+{
+    const auto mappedIndex = m_listFilter->mapFromSource(index);
+    scrollTo(mappedIndex);
+    clearSelection();
+    selectionModel()->select(mappedIndex, QItemSelectionModel::Select);
+}
