@@ -1,7 +1,6 @@
-/* SPDX-FileCopyrightText: 2020-2021 Tobias Leupold <tl@l3u.de>
-
-   SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-KDE-Accepted-GPL
-*/
+// SPDX-FileCopyrightText: 2020-2022 Tobias Leupold <tl at stonemx dot de>
+//
+// SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
 // Local includes
 #include "MainWindow.h"
@@ -53,6 +52,10 @@
 #include <QAbstractButton>
 #include <QVBoxLayout>
 
+// C++ includes
+#include <functional>
+#include <algorithm>
+
 static const QHash<QString, KExiv2Iface::KExiv2::MetadataWritingMode> s_writeModeMap {
     { QStringLiteral("WRITETOIMAGEONLY"),
       KExiv2Iface::KExiv2::MetadataWritingMode::WRITETOIMAGEONLY },
@@ -72,6 +75,7 @@ MainWindow::MainWindow(SharedObjects *sharedObjects)
       m_geoDataModel(sharedObjects->geoDataModel())
 {
     setWindowTitle(i18n("KGeoTag"));
+    setWindowIcon(QIcon::fromTheme(QStringLiteral("kgeotag")));
 
     connect(m_elevationEngine, &ElevationEngine::elevationProcessed,
             this, &MainWindow::elevationProcessed);
@@ -88,13 +92,15 @@ MainWindow::MainWindow(SharedObjects *sharedObjects)
     addFilesAction->setText(i18n("Add images and/or GPX tracks"));
     addFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("document-new")));
     actionCollection()->setDefaultShortcut(addFilesAction, QKeySequence(tr("Ctrl+F")));
-    connect(addFilesAction, &QAction::triggered, this, &MainWindow::addFiles);
+    connect(addFilesAction, &QAction::triggered,
+            this, std::bind(&MainWindow::addFiles, this, QStringList()));
 
     auto *addDirectoryAction = actionCollection()->addAction(QStringLiteral("addDirectory"));
-    addDirectoryAction->setText(i18n("Add all images and tracks from directory"));
+    addDirectoryAction->setText(i18n("Add all images and tracks from a folder"));
     addDirectoryAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-insert-directory")));
     actionCollection()->setDefaultShortcut(addDirectoryAction, QKeySequence(tr("Ctrl+D")));
-    connect(addDirectoryAction, &QAction::triggered, this, &MainWindow::addDirectory);
+    connect(addDirectoryAction, &QAction::triggered,
+            this, std::bind(&MainWindow::addDirectory, this, QString()));
 
     // "Remove" submenu
 
@@ -298,6 +304,8 @@ QDockWidget *MainWindow::createImagesDock(KGeoTag::ImagesListType type, const QS
     connect(list, &ImagesListView::requestAddingImages, this, &MainWindow::addImages);
     connect(list, &ImagesListView::removeImages, this, &MainWindow::removeImages);
     connect(list, &ImagesListView::requestSaving, this, &MainWindow::saveSelection);
+    connect(list, &ImagesListView::failedToParseClipboard,
+            this, &MainWindow::failedToParseClipboard);
 
     return createDockWidget(title, list, dockId);
 }
@@ -401,20 +409,54 @@ void MainWindow::closeEvent(QCloseEvent *event)
     QApplication::quit();
 }
 
-void MainWindow::addFiles()
+void MainWindow::addPathsFromCommandLine(QStringList &paths)
 {
-    const auto selection = QFileDialog::getOpenFileNames(this,
-                                i18n("Please select the images and/or GPX tracks to add"),
-                                m_settings->lastOpenPath(),
-                                i18n("All supported files ("
-                                    "*.jpg *.jpeg "
-                                    "*.png "
-                                    "*.webp "
-                                    "*.tif *.tiff "
-                                    "*.ora "
-                                    "*.kra "
-                                    "*.gpx "
-                                    ");; All files (*)"));
+    QVector<QString> directories;
+    QStringList files;
+
+    paths.removeDuplicates();
+    for (const auto &path : std::as_const(paths)) {
+        const QFileInfo info(path);
+        if (info.isDir()) {
+            directories.append(path);
+        } else {
+            files.append(path);
+        }
+    }
+
+    QApplication::processEvents();
+
+    if (! directories.isEmpty()) {
+        for (const auto &directory : std::as_const(directories)) {
+            addDirectory(directory);
+        }
+    }
+
+    if (! files.isEmpty()) {
+        addFiles(files);
+    }
+}
+
+void MainWindow::addFiles(const QStringList &files)
+{
+    QStringList selection;
+
+    if (files.isEmpty()) {
+        selection = QFileDialog::getOpenFileNames(this,
+                        i18n("Please select the images and/or GPX tracks to add"),
+                        m_settings->lastOpenPath(),
+                        i18n("All supported files ("
+                                 "*.jpg *.jpeg "
+                                 "*.png "
+                                 "*.webp "
+                                 "*.tif *.tiff "
+                                 "*.ora "
+                                 "*.kra "
+                                 "*.gpx "
+                             ");; All files (*)"));
+    } else {
+        selection = files;
+    }
 
     if (selection.isEmpty()) {
         return;
@@ -470,11 +512,16 @@ void MainWindow::addFiles()
     }
 }
 
-void MainWindow::addDirectory()
+void MainWindow::addDirectory(const QString &path)
 {
-    const auto directory = QFileDialog::getExistingDirectory(this,
-                               i18n("Please select a directory"),
-                               m_settings->lastOpenPath());
+    QString directory;
+
+    if (path.isEmpty()) {
+        directory = QFileDialog::getExistingDirectory(this, i18n("Please select a folder"),
+                                                      m_settings->lastOpenPath());
+    } else {
+        directory = path;
+    }
 
     if (directory.isEmpty()) {
         return;
@@ -500,7 +547,7 @@ void MainWindow::addDirectory()
     }
 
     if (geoDataFiles.isEmpty() && images.isEmpty()) {
-        QMessageBox::warning(this, i18n("Add all images and tracks from directory"),
+        QMessageBox::warning(this, i18n("Add all images and tracks from a folder"),
                              i18n("Could not find any supported files in <kbd>%1</kbd>",
                                   directory));
         return;
@@ -868,10 +915,16 @@ void MainWindow::addImages(const QVector<QString> &paths)
 
 void MainWindow::imagesDropped(const QVector<QString> &paths)
 {
-    m_previewWidget->setImage(m_imagesModel->indexFor(paths.last()));
+    const auto index = m_imagesModel->indexFor(paths.last());
+
+    m_previewWidget->setImage(index);
     if (m_settings->lookupElevationAutomatically()) {
         lookupElevation(paths);
     }
+
+    if (m_settings->splitImagesList()) {
+        qobject_cast<ImagesListView *>(m_assignedOrAllImagesDock->widget())->highlightImage(index);
+    }
 }
 
 void MainWindow::assignToMapCenter(ImagesListView *list)
@@ -946,6 +999,11 @@ void MainWindow::assignTo(const QVector<QString> &paths, const Coordinates &coor
     if (m_settings->lookupElevationAutomatically()) {
         lookupElevation(paths);
     }
+
+    if (m_settings->splitImagesList()) {
+        qobject_cast<ImagesListView *>(m_assignedOrAllImagesDock->widget())->highlightImage(
+            m_imagesModel->indexFor(paths.last()));
+    }
 }
 
 void MainWindow::triggerAutomaticMatching(ImagesListView *list, KGeoTag::SearchType searchType)
@@ -1651,3 +1709,10 @@ void MainWindow::centerTrackPoint(int trackIndex, int trackPointIndex)
     m_mapCenterInfo->trackPointCentered(coordinates, dateTime);
     m_mapWidget->blockSignals(false);
 }
+
+void MainWindow::failedToParseClipboard()
+{
+    QMessageBox::warning(this,
+                         i18n("Failed to parse clipboard data"),
+                         i18n("Could not parse the clipboard's text to valid coordinates"));
+}
