@@ -1,7 +1,6 @@
-/* SPDX-FileCopyrightText: 2020-2021 Tobias Leupold <tl@l3u.de>
-
-   SPDX-License-Identifier: GPL-3.0-or-later OR LicenseRef-KDE-Accepted-GPL
-*/
+// SPDX-FileCopyrightText: 2020-2022 Tobias Leupold <tl at stonemx dot de>
+//
+// SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 
 // Local includes
 #include "MainWindow.h"
@@ -16,7 +15,7 @@
 // Qt includes
 #include <QApplication>
 #include <QDebug>
-#include <QDate>
+#include <QCommandLineParser>
 
 int main(int argc, char *argv[])
 {
@@ -35,15 +34,25 @@ int main(int argc, char *argv[])
     aboutData.setVersion(VERSION_STRING);
     aboutData.setShortDescription(i18n("Photo geotagging program"));
     aboutData.setLicense(KAboutLicense::GPL_V3);
-    aboutData.setCopyrightStatement(i18n("Copyright (C) 2020-2021 Tobias Leupold"));
+    aboutData.setCopyrightStatement(i18n("Copyright (C) 2020-2022 Tobias Leupold"));
     aboutData.setHomepage(QStringLiteral("https://kgeotag.kde.org/"));
     aboutData.setOrganizationDomain(QStringLiteral("kde.org").toUtf8());
 
     aboutData.addAuthor(i18n("Tobias Leupold"), i18n("Maintainer"),
-                        QStringLiteral("tl@l3u.de"));
+                        QStringLiteral("tl@stonemx.de"));
 
     KAboutData::setApplicationData(aboutData);
 
+    // Create the command line parser
+    QCommandLineParser commandLineParser;
+    commandLineParser.addPositionalArgument(i18n("Files/Folders"),
+                                            i18n("Files and/or folders to load at startup"),
+                                            i18n("[files/folders...]"));
+    aboutData.setupCommandLine(&commandLineParser);
+    commandLineParser.process(application);
+    aboutData.processCommandLine(&commandLineParser);
+    auto pathsToLoad = commandLineParser.positionalArguments();
+
     // Setup all shared objects
     SharedObjects sharedObjects;
 
@@ -51,6 +60,11 @@ int main(int argc, char *argv[])
     auto *mainWindow = new MainWindow(&sharedObjects);
     mainWindow->show();
 
-    // Run the program
+    // Trigger loading of files and/or directories given on the command line
+    if (! pathsToLoad.isEmpty()) {
+        mainWindow->addPathsFromCommandLine(pathsToLoad);
+    }
+
+    // Run the QApplication
     return application.exec();
 }
